<?php
namespace Teknisa\FIN\Modules\ContaReceber\Services\Remessa\Banco;

use Financeiro\Entities\Boletreceber;
use Financeiro\Entities\Contareceber;
use Financeiro\Entities\Emprbanc;
use Financeiro\Util\Util;
use Geral\Entities\Banco;
use Geral\Entities\Cliente;
use Geral\Entities\Empresa;
use Geral\Entities\Endeclie;
use Geral\Entities\Municipio;
use Teknisa\Libs\Util\DateUtil;
use Doctrine\ORM\EntityManager;
use Geral\Entities\Contcorr;
use Geral\Entities\Filial;
use Teknisa\FIN\Modules\ContaReceber\Services\Remessa\GeraCnabService;
use Teknisa\FIN\Modules\ContaReceber\Factory\TitulosAReceberFactory;

/**
 * Class GeraCnabVotorantimService
 * @package Teknisa\FIN\Modules\ContaReceber\Services\Remessa\Banco
 */
class GeraCnabVotorantimService extends GeraCnabService{
    
    const TIPO_INSCRICAO_CEDENTE_CNPJ = '2';
    const TIPO_INSCRICAO_CEDENTE_CPF = '1';
    const CODIGO_BANCO = '655';
    const LOTE_SERVICO = '0000';
    const TIPO_REGISTRO = '0';
    const CODIGO_REMESSA = '1';
    const VERSAO_LAYOUT_ARQUIVO = '080';
    const LOTE_SERVICO_HEADER_LOTE = '0001';
    const TIPO_REGISTRO_HEADER_LOTE = '1';
    const TIPO_OPERACAO = 'R';
    const TIPO_SERVICO_HEADER_LOTE = '01';
    const VERSAO_LAYOUT_HEADER_LOTE = '040';
    const TIPO_REGISTRO_SEGMENTOS = '3';
    const CODIGO_SEGMENTO_P = 'P';
    const ID_OPERACAO_CNAB_INCLUSAO = '01';
    const ID_OPERACAO_CNAB_EXCLUSAO = '02';
    const IDENTIFICACAO_REGISTRO_R = '3';
    const IDENTIFICACAO_SEGMENTO_REGISTRO_R = 'R';
    const IDENTIFICACAO_SEGMENTO_REGISTRO_Q = 'Q';
    const IDENTIFICACAO_REGISTRO_Q = '3';
    const ID_OPERACAO_CNAB_ALTERACAO ='06';
    const TIPO_REGISTRO_TRAILLER_LOTE ='5';
    const IDENTIFICACAO_ARQUIVO_TRAILLER ='9';
    const LOTE_SERVICO_TRAILLER ='9999';
    const CODIGO_MOVIMENTO ='01';
    const DENSIDADE ='01600';
    const LOTE_DO_SERVICO = '0001';
    const TIPO_DOCUMENTO = '2';
    const ESPECIE_TITULO = '02';
    const ID_TITULO_ACEITO = 'N';
    const CODIGO_DESCONTO = '0';
    const CODIGO_BAIXA = '2';
    const CODIGO_MOEDA = '09';
    const LOTE_SERVICO_2 = '0001';

    
    /**
     * Gera Header Arquivo
     * @param $caminho
     * @param $nmarquivo
     * @param $nrorg
     * @param $cdbanco
     * @param $cdempresa
     * @param $cdidrec
     * @param $nrseqremessarec
     * @param $nrcnab
     * @param $cdreducont
     * @param null $carteira
     * @return mixed|void
     */
    public function geraHeaderArquivo(
        $caminho, $nmarquivo, $nrorg, $cdbanco, $cdempresa, $cdidrec, $nrseqremessarec,
        $nrcnab, $cdreducont, $carteira = NULL, $contreg = NULL, $idtipcnabrec = NULL, $cdfilial, $contador = NULL
    ){
      
      if (!is_dir($caminho))
        mkdir($caminho, "0777", true);
        $nmarquivo = $caminho.$nmarquivo;
        $file = fopen($nmarquivo, "w+");
        
      //Descrição: Código do Banco na Compensação
      //Posição: 1 a 3
      //Tamanho: X(3)
      $arquivo = Util::campoCnabValido(self::CODIGO_BANCO);
      
      //Descrição: Lote de Serviço
      //Posição: 4 a 7
      //Tamanho: X(4)
      $arquivo .= Util::campoCnabValido(self::LOTE_SERVICO);
      
      //Descrição: Tipo de Registro
      //Posição: 8 a 8
      //Tamanho: X(1)
      //'0' = Header de Arquivo
      $arquivo .= Util::campoCnabValido(self::TIPO_REGISTRO);
      
      //Descrição: Uso Exclusivo FEBRABAN / CNAB
      //Posição: 9 a 17
      //Tamanho: X(9)
      //Brancos
      $arquivo .= self::preenche_brancos('', 9);
      
      //Descrição: Tipo de Inscrição da Empresa
      //Posição: 18 a 18
      //Tamanho: X(1)
      //'1' = CPF - '2' = CGC / CNPJ
      $nrinsjurempr = Empresa::findOneByNrorgAndCdempresa($nrorg, $cdempresa);
      $idtpijurempr = $nrinsjurempr->getNrinsjurempr();
      
      if (strlen($idtpijurempr) == '14'){
        $stTpInscCedente = self::TIPO_INSCRICAO_CEDENTE_CNPJ; //CNPJ
      }
      else{
        $stTpInscCedente = self::TIPO_INSCRICAO_CEDENTE_CPF; //CPF
      }
      
      $arquivo .= Util::campoCnabValido($stTpInscCedente);
      
      //Descrição: Número de Inscrição da Empresa
      //Posição: 19 a 32
      //Tamanho: X(14)
      $nrinsjurempr = Empresa::findOneByNrorgAndCdempresa($nrorg, $cdempresa);
       
       // Verifica se foi informado filial na geração do CNAB, caso seja, sera passado o NRINSJUR da filial e não da empresa.
      if($cdfilial) {
          
          $filial = Filial::findOneByCdfilial($cdfilial);
          
          if ($filial) {
              $nrinsjuremp = Util::campoCnabValido($filial->getNrinsjurfili());
          } else {
              $nrinsjurempr = Util::campoCnabValido($nrinsjurempr->getNrinsjurempr());
          }
          
          $arquivo .= Util::campoCnabValido(str_pad($nrinsjurempr, 14, "0", STR_PAD_LEFT));
         
      } else{
          $arquivo .= Util::campoCnabValido(str_pad($nrinsjurempr->getNrinsjurempr(), 14, "0", STR_PAD_LEFT));
      }
    
      //Descrição: Código do Convênio no Banco
      //Posição: 33 a 52
      //Tamanho: X(20)
      $arquivo .= self::preenche_brancos('',20);
      
      //Descrição: Agência Mantenedora da Conta
      //Posição: 53 a 57
      //Tamanho: X(5)
      $contcorr = Contcorr::findOneByCdreducont($cdreducont);
      if($contcorr && $contcorr->getCdagencia())
          $arquivo .= str_pad(substr($contcorr->getCdagencia(), 0, 4), 5, '0', STR_PAD_LEFT);
      else
          $arquivo .= str_repeat('0',5);
      
      //Descrição: Dígito Verificador da Agência
      //Posição: 58 a 58
      //Tamanho: X(1) 
      $arquivo .= self::preenche_brancos('', 1);
      
      //Descrição: Número da Conta Corrente
      //Posição: 59 a 70
      //Tamanho: X(12)
      if($contcorr && $contcorr->getCdcontcorr()){
        $tamanho = strlen(str_replace(' ', '', $contcorr->getCdcontcorr())) - 1;
        $arquivo .= str_pad(substr(str_replace(' ', '', $contcorr->getCdcontcorr()), 0, $tamanho), 12, '0', STR_PAD_LEFT);
        
      } else {
        $arquivo .= str_repeat('0',12);
      }
     
      //Descrição: Dígito Verificador da Conta
      //Posição:  71 a 71
      //Tamanho: X(1)
      if($contcorr) {
           $arquivo .= substr(str_replace(' ', '', $contcorr->getCdcontcorr()), -1);
      } else {
        $arquivo .= self::preenche_brancos('', 1);
      }
      
      //Descrição: Dígito Verificador da Ag/Conta
      //Posição:  72 a 72
      //Tamanho: X(1)
      $arquivo .= self::preenche_brancos('', 1);
      
      //Descrição: Nome da Empresa
      //Posição:  73 a 102
      //Tamanho: X(30)
      $nomeempresa = Empresa::findOneByNrorgAndCdempresa($nrorg, $cdempresa);
      
      // Verifica se foi informado filial na geração do CNAB, caso seja, sera passado o nome da filial e não da empresa.
      $nmempresa = substr($nomeempresa->getNmempresa(), 0, 30);
      
      if($cdfilial) {
          $filial = Filial::findOneByCdfilial($cdfilial);
          if($filial) {
              $nmempresa = substr($filial->getNmrazsocfili(), 0, 30);
          } else {
              $nmempresa = substr($nomeempresa->getNmempresa(), 0, 30);
          }
          
          $arquivo .= Util::campoCnabValido(str_pad($nmempresa, 30));
      } else {
          $arquivo .= Util::campoCnabValido(str_pad($nmempresa, 30));
      }
      
      //Descrição: Nome do Banco
      //Posição:  103 a 132
      //Tamanho: X(30)
      $nmbanco = Banco::getObjectByWhere('CDBANCO = ? AND NRORG = ?', array($cdbanco, $nrorg));
      $arquivo .= Util::campoCnabValido(str_pad($nmbanco->getNmbanco(), 30));
      
      //Descrição: Uso Exclusivo FEBRABAN / CNAB - Brancos
      //Posição:  133 a 142
      //Tamanho: X(10)
      $arquivo .= self::preenche_brancos('', 10);
       
      //Descrição: Código Remessa / Retorno
      //Posição:  143 a 143
      //Tamanho: X(1)
      //Domínio: '1' = Remessa (Cliente > Banco) / '2' = Retorno (Banco > Cliente)
      $arquivo .= Util::campoCnabValido(self::CODIGO_REMESSA);
      
      //Descrição: Data de Geração do Arquivo
      //Posição:  144 a 151
      //Tamanho: X(8)
      $now = DateUtil::getDataAtual();
      $data = DateUtil::getDia($now) . DateUtil::getMes($now) . DateUtil::getAno($now);
      $arquivo .= Util::campoCnabValido($data);
      
      //Descrição: Hora de Geração do Arquivo
      //Posição:  152 a 157
      //Tamanho: X(6)
      $now = DateUtil::getDataAtual();
      $horaGeracao = $now->format('His');
      $arquivo .= $horaGeracao;
      
      //Descrição: Número Seqüencial do Arquivo
      //Posição:  158 a 163
      //Tamanho: X(6)
      $arquivo .= Util::campoCnabValido(substr($nrseqremessarec, -6));
      
      //Descrição: No da Versão do Layout do Arquivo
      //Posição:  164 a 166
      //Tamanho: X(3)
      // FIXO 080'
      $arquivo .= Util::campoCnabValido(self::VERSAO_LAYOUT_ARQUIVO);
      
      //Descrição: Densidade de Gravação do Arquivo
      //Posição:  167 a 171
      //Tamanho: X(5)
      //Domínio: 1600 BPI / 6250 BPI
      $arquivo .= Util::campoCnabValido(self::DENSIDADE);
      
      //Descrição: Para Uso Reservado do Banco
      //Posição:  172 a 191
      //Tamanho: X(20)
      $arquivo .= self::preenche_brancos('', 20);
      
      //Descrição: Para Uso Reservado da Empresa
      //Posição:  192 a 211
      //Tamanho: X(20)
      $arquivo .= self::preenche_brancos('', 20);
      
      //Descrição: Uso Exclusivo FEBRABAN / CNAB
      //Posição:  212 a 240
      //Tamanho: X(29)
      //Brancos
      $arquivo .= self::preenche_brancos('', 29)  . "\r\n";
      
      //Grava a variavel arquivo no arquivo texto
      fprintf($file, "%s", $arquivo);
      fclose($file);
      
    }

    /**
     * Gera Header Lote
     * @param $caminho
     * @param $nmarquivo
     * @param $nrorg
     * @param $cdbanco
     * @param $cdempresa
     * @param $cdidrec
     * @param $nrseqremessarec
     * @param $nrcnabrec
     * @param $cdreducont
     * @param null $carteira
     * @return mixed|void
     */
    public function geraHeaderLote(
        $caminho, $nmarquivo, $nrorg, $cdbanco, $cdempresa, $cdidrec, $nrseqremessarec,
        $nrcnabrec, $cdreducont, $carteira = NULL, $cdfilial
    ){
      
     if (!is_dir($caminho))
          mkdir($caminho, "0777", true);
      $nmarquivo = $caminho.$nmarquivo;
      $file = fopen($nmarquivo, "a+");

      //Descrição: Código do Banco na Compensação
      //Posição: 1 a 3
      //Tamanho: X(3)
      $banco = Banco::getObjectByWhere('CDBANCO = ? AND NRORG = ?', array($cdbanco, $nrorg));
      $arquivo = $banco->getCdbanccnab();
      
      //Descrição: Lote de Serviço
      //Posição: 4 a 7
      //Tamanho: X(4)
      $arquivo .= Util::campoCnabValido(self::LOTE_SERVICO_HEADER_LOTE);
      
      //Descrição: Tipo de Registro
      //Posição: 8 a 8
      //Tamanho: X(1)
      // FIXO 1
      $arquivo .= Util::campoCnabValido(self::TIPO_REGISTRO_HEADER_LOTE);
      
      //Descrição: Tipo de Operação 
      //Posição: 9 a 9
      //Tamanho: X(1)
      /**Domínio: 
       * 'C' = Lançamento a Crédito - 
       * 'D' = Lançamento a Débito
       * 'E' = Extrato para Conciliação 
       * 'G' = Extrato para Gestão de Caixa
       * 'I' = Informações de Títulos Capturados do Próprio Banco - 
       * 'R' = Arquivo Remessa - 
       * 'T' = Arquivo Retorno
      */
      $arquivo .= Util::campoCnabValido(self::TIPO_OPERACAO);
      
      //Descrição: Tipo de Serviço
      //Posição: 10 a 11
      //Tamanho: X(2)
      // FIXO 01
      $arquivo .= Util::campoCnabValido(self::TIPO_SERVICO_HEADER_LOTE);
      
      //Descrição: Uso Exclusivo FEBRABAN/CNAB 
      //Posição: 12 a 13
      //Tamanho: X(2)
      //Brancos
      $arquivo .= self::preenche_brancos('', 2);
      
      //Descrição: Nº da Versão do Layout do Lote
      //Posição: 14 a 16
      //Tamanho: X(3)
      // FIXO 040
      $arquivo .= Util::campoCnabValido(self::VERSAO_LAYOUT_HEADER_LOTE);
      
      //Descrição: Uso Exclusivo FEBRABAN/CNAB
      //Posição: 17 a 17
      //Tamanho: X(1)
      //Brancos
      $arquivo .= self::preenche_brancos('', 1);
      
      //Descrição: Tipo de Inscrição da Empresa
      //Posição: 18 a 18
      //Tamanho: X(1)
      $nrinsjurempr = Empresa::findOneByNrorgAndCdempresa($nrorg, $cdempresa);
      $idtpijurempr = $nrinsjurempr->getNrinsjurempr();
      if (strlen($idtpijurempr) == '14'){
          $stTpInscCedente = '2'; //CNPJ
      }
      else{
          $stTpInscCedente = '1'; //CPF
      }
      $arquivo .= Util::campoCnabValido($stTpInscCedente);
      
      //Descrição: Nº de Inscrição da Empresa 
      //Posição: 19 a 33
      //Tamanho: X(15)
      $nrinsjurempr = Empresa::findOneByNrorgAndCdempresa($nrorg, $cdempresa);

      // Verifica se foi informado filial na geração do CNAB, caso seja, sera passado o NRINSJUR da filial e não da empresa.
      if($cdfilial) {
          $filial = Filial::findOneByCdfilial($cdfilial);
          
          if ($filial) {
              $nrinsjurempr = Util::campoCnabValido($filial->getNrinsjurfili());
          } else {
              $nrinsjurempr = Util::campoCnabValido($nrinsjurempr->getNrinsjurempr());
          }
          $arquivo .= Util::campoCnabValido(str_pad($nrinsjurempr, 15, "0", STR_PAD_LEFT));
      } else {
          $arquivo .= Util::campoCnabValido(str_pad($nrinsjurempr->getNrinsjurempr(), 15, "0", STR_PAD_LEFT));
      }
      
      //Descrição: Código do Convênio no Banco
      //Posição: 34 a 53
      //Tamanho: X(20)
      $arquivo .= self::preenche_brancos('',20);
      
      //Descrição: Agência Mantenedora da Conta 
      //Posição: 54 a 58
      //Tamanho: X(5)
      $contcorr = Contcorr::findOneByCdreducont($cdreducont);
      if($contcorr && $contcorr->getCdagencia())
          $arquivo .= str_pad(substr($contcorr->getCdagencia(), 0, 4), 5, '0', STR_PAD_LEFT);
      else
         $arquivo .= str_repeat('0',5);
      
      //Descrição: Dígito Verificador da Conta
      //Posição: 59 a 59
      //Tamanho: X(1)
      if($contcorr)
        $arquivo .= substr($contcorr->getCdagencia(), -1);
      ; 
      //Descrição: Número da Conta Corrente
      //Posição: 60 a 71
      //Tamanho: X(12)
     if($contcorr && $contcorr->getCdcontcorr()){
          $tamanho = strlen(str_replace(' ', '', $contcorr->getCdcontcorr())) - 1;
          $arquivo .= str_pad(substr(str_replace(' ', '', $contcorr->getCdcontcorr()), 0, $tamanho), 12, '0', STR_PAD_LEFT);
      } else {
          $arquivo .= str_repeat('0',12);
      }
       
      //Descrição: Dígito Verificador da Conta
      //Posição: 72 a 72
      //Tamanho: X(1)
      if($contcorr){
          $arquivo .= substr(str_replace(' ', '', $contcorr->getCdcontcorr()), -1);
      } else {
        $arquivo .= self::preenche_brancos('', 1);
      }
      
      //Descrição: Dígito Verificador da Ag/Conta
      //Posição: 73 a 73
      //Tamanho: X(1)
      $arquivo .= self::preenche_brancos('', 1);
      
      //Descrição: Nome da Empresa
      //Posição: 74 a 103
      //Tamanho: X(30)
      $empresa = Empresa::findOneByCdempresa($cdempresa);
      $filial = Filial::findOneByCdfilial($cdfilial);
      if($filial) {
          $nmrazsocemp = Util::campoCnabValido($filial->getNmrazsocfili());
          $nmrazsocemp = substr($nmrazsocemp, 0, 30);
          $arquivo .= str_pad($nmrazsocemp, 30);
      } else {
          $nmrazsocemp = Util::campoCnabValido($empresa->getNmrazsocempr());
          $nmrazsocemp = substr($nmrazsocemp, 0, 30);
          $arquivo .= str_pad($nmrazsocemp, 30);
      }
      
      //Descrição: Mensagem 1
      //Posição: 104 a 143
      //Tamanho: X(40)
      $arquivo .= self::preenche_brancos('', 40);
      
      //Descrição: Mensagem 2 
      //Posição: 144 a 183
      //Tamanho: X(40)
      $arquivo .= self::preenche_brancos('', 40);
      
      //Descrição: Número Remessa/Retorno 
      //Posição: 184 a 191
      //Tamanho: X(8)
      $arquivo .= Util::campoCnabValido(substr($nrcnabrec, -8));
      
      //Descrição: Data de Gravação Remessa/Retorno
      //Posição: 192 a 199
      //Tamanho: X(8)
      $now = DateUtil::getDataAtual();
      $data = DateUtil::getDia($now) . DateUtil::getMes($now) . DateUtil::getAno($now);
      $arquivo .= Util::campoCnabValido($data);
      
      //Descrição: Data do Crédito
      //Posição: 200 a 207
      //Tamanho: X(8)
      $arquivo .= str_repeat('0',8);
      
      //Descrição: Uso Exclusivo FEBRABAN/CNAB 
      //Posição: 208 a 240
      //Tamanho: X(33)
      //Brancos
      $arquivo .= self::preenche_brancos('', 33). "\r\n";

      //Grava a variavel arquivo no arquivo texto
      fprintf($file, "%s", $arquivo);
      fclose($file);
      
      
    }

    /**
     * Gera Detalhe Segmento P
     * @param $caminho
     * @param $nmarquivo
     * @param $nrorg
     * @param $cdbanco
     * @param $cdempresa
     * @param $cdidrec
     * @param $nrseqremessarec
     * @param $nrcnabrec
     * @param $idoperacao
     * @param $cdagenciacnab
     * @param $cdcontacorrente
     * @param $cdcontcorrcobranca
     * @param $nossonumero
     * @param $idtipocobranca
     * @param $nrlanbolrec
     * @param $cdfilial
     * @param $dtvenbolrec
     * @param $valorfinalboleto
     * @param $valorpemobrec
     * @param $valormordiabrec
     * @param $valorpemulbrec
     * @param $valormultabrec
     * @param $valorfinalbrec
     * @param $cdcartcobbol
     * @param $cddiaprotbol
     * @param $cdreducont
     * @param $imprbol
     * @param $qtdiaprotbol
     * @param $contador
     * @return mixed|void
     */
    public function geraDetalheSegmentoP(
        $caminho, $nmarquivo, $nrorg, $cdbanco, $cdempresa, $cdidrec, $nrseqremessarec, $nrcnabrec,
        $idoperacao, $cdagenciacnab, $cdcontacorrente, $cdcontcorrcobranca, $nossonumero, $idtipocobranca,
        $nrlanbolrec, $cdfilial, $dtvenbolrec, $valorfinalboleto, $valorpemobrec, $valormordiabrec, $valorpemulbrec,
        $valormultabrec, $valorfinalbrec, $cdcartcobbol, $cddiaprotbol, $cdreducont, $imprbol, $qtdiaprotbol, $contador = NULL
    ){
        
      if (!is_dir($caminho))
          mkdir($caminho, "0777", true);
      $nmarquivo = $caminho.$nmarquivo;
      $file = fopen($nmarquivo, "a+");
        
      //Descrição: Código do Banco na Compensação 
      //Posição: 1 a 3
      //Tamanho: X(3)
      $banco = Banco::getObjectByWhere('CDBANCO = ? AND NRORG = ?', array($cdbanco, $nrorg));
      $arquivo = $banco->getCdbanccnab();
      
      //Descrição: Lote de Serviço
      //Posição: 4 a 7
      //Tamanho: X(4)
      $arquivo .= Util::campoCnabValido(self::LOTE_DO_SERVICO);
      
      //Descrição: Tipo de Registro
      //Posição: 8 a 8
      //Tamanho: X(1)
      $arquivo .= Util::campoCnabValido(self::TIPO_REGISTRO_SEGMENTOS);
      
      //Descrição: Nº Sequencial do Registro no Lote
      //Posição: 9 a 13
      //Tamanho: X(5)
      $contador = str_pad($contador, 5, "0", STR_PAD_LEFT);
      $arquivo .= Util::campoCnabValido($contador);
      
      //Descrição: Cód. Segmento do Registro Detalhe
      //Posição: 14 a 14
      //Tamanho: X(1)
      $arquivo .= Util::campoCnabValido(self::CODIGO_SEGMENTO_P);
      
      //Descrição: Uso Exclusivo FEBRABAN/CNAB
      //Posição: 15 a 15
      //Tamanho: X(1)
      //Brancos
      $arquivo .= self::preenche_brancos('', 1);
      
      //Descrição: Código de Movimento Remessa
      //Posição: 16 a 17
      //Tamanho: X(2)
      $idopecnabrec = $idoperacao;
        if((bool) $idopecnabrec !== false){
          if($idopecnabrec == 'I')
                $arquivo .= Util::campoCnabValido(self::ID_OPERACAO_CNAB_INCLUSAO); //Inclusão
          else if($idopecnabrec == 'E')
                $arquivo .= Util::campoCnabValido(self::ID_OPERACAO_CNAB_EXCLUSAO); //Exclusão
          else
                $arquivo .= Util::campoCnabValido(self::ID_OPERACAO_CNAB_ALTERACAO); //Alteração
        }
      
      //Descrição: Agência Mantenedora da Conta
      //Posição: 18 a 22
      //Tamanho: X(5)
      $contcorr = Contcorr::findOneByCdreducont($cdreducont);
      if($contcorr && $contcorr->getCdagencia())
          $arquivo .= str_pad(substr($contcorr->getCdagencia(), 0, 4), 5, '0', STR_PAD_LEFT);
      else
         $arquivo .= str_repeat('0',5);
      
         
      //Descrição: Dígito Verificador da Agência
      //Posição: 23 a 23
      //Tamanho: X(1)
      if($contcorr)
        $arquivo .= substr($contcorr->getCdagencia(), -1);
      else 
        $arquivo .= self::preenche_brancos('', 1);
        
      
      //Descrição: Número da Conta Corrente
      //Posição: 24 a 35
      //Tamanho: X(12)
      if($contcorr && $contcorr->getCdcontcorr()){
          $tamanho = strlen(str_replace(' ', '', $contcorr->getCdcontcorr())) - 1;
          $arquivo .= str_pad(substr(str_replace(' ', '', $contcorr->getCdcontcorr()), 0, $tamanho), 12, '0', STR_PAD_LEFT);
      }
      else
          $arquivo .= str_repeat('0',12);

      //Descrição: Dígito Verificador da Conta
      //Posição: 36 a 36
      //Tamanho: X(1)
      if($contcorr) {
           $arquivo .= substr(str_replace(' ', '', $contcorr->getCdcontcorr()), -1);
      } else {
        $arquivo .= self::preenche_brancos('', 1);
      }
      
      //Descrição: Dígito Verificador da Ag/Conta
      //Posição: 37 a 37
      //Tamanho: X(1)
      $arquivo .= self::preenche_brancos('', 1);
      
      //Descrição: Zeros
      //Posição: 38 a 47
      //Tamanho: X(10)
      $arquivo .= str_repeat('0',10);
      
      //Descrição: Identificação do Título no Banco
      //Posição: 48 a 57
      //Tamanho: X(10)
      $entityBoleto = Boletreceber::getRepository();
      $duplicata = $entityBoleto->getDuplicata($cdfilial, $nrlanbolrec);
      
      if (!$duplicata[0]['DUPLICATA'])
          $arquivo .= str_pad($nrlanbolrec, 10);
      else {
          $arquivo .= str_pad($duplicata[0]['DUPLICATA'].$duplicata[0]['NRPARCELA'],10,0,STR_PAD_LEFT);
      }

      //Descrição: Código da Carteira
      //Posição: 58 a 58
      //Tamanho: X(1)
      /**
       * '1' = Cobrança Simples
       * '3' = Cobrança Garantia
       '4' = Cobrança Descontada
       */
       $boleto = Boletreceber::findOneByNrlanbolrecAndCdfilial($nrlanbolrec, $cdfilial);
       switch($boleto->getIdcartcobrec()){
            case '1':
                $arquivo .= '1';
                break;
            case '3':
                $arquivo .= '3';
                break;
            case '4':
                $arquivo .= '4';
                break;
        }
      
      //Descrição: Forma de Cadastr. do Título no Banco
      //Posição: 59 a 59
      //Tamanho: X(1)
      /**
       * '1' = Com Cadastramento (Cobrança Registrada)
       * '2' = Sem Cadastramento (Cobrança sem Registro)
       * Obs.: Destina-se somente para emissão de bloqueto pelo banco
      */
      $boleto = Boletreceber::findOneByNrlanbolrecAndCdfilial($nrlanbolrec, $cdfilial);
        if($boleto && $boleto->getIdformcadbrec())
            $arquivo .= $boleto->getIdformcadbrec();
        else
            $arquivo .= '1';
      
      //Descrição: Tipo de Documento
      //Posição: 60 a 60
      //Tamanho: X(1)
      /**
       * '1' = Tradicional
       * '2' = Escritural
       */
       $arquivo .= self::TIPO_DOCUMENTO;
       
      //Descrição: Identificação da Emissão do Bloqueto
      //Posição: 61 a 61
      //Tamanho: X(1)
      /**
       * '1' = Banco Emite
       * '2' = Cliente Emite
       */
       if($imprbol == 'B')
          $arquivo .= '1';
       else
          $arquivo .= '2';
      
      //Descrição: Identificação da Distribuição
      //Posição: 62 a 62
      //Tamanho: X(1)
      /**
       * '1' = Banco Distribui
       * '2' = Cliente Distribui
       * ‘3’ = Banco envia e-mail
       * ‘4’ = Banco envia SMS
       */
      if($imprbol == 'B')
          $arquivo .= '2';
      else
          $arquivo .= '1';
          
      //Descrição: Número do Documento de Cobrança
      //Posição: 63 a 72
      //Tamanho: X(10)
      $entityBoleto = Boletreceber::getRepository();
      $duplicata = $entityBoleto->getDuplicata($cdfilial, $nrlanbolrec);
      if (!$duplicata[0]['DUPLICATA'])
          $arquivo .= str_pad($nrlanbolrec, 10, '0');
      else {
          $arquivo .= str_pad(substr($duplicata[0]['DUPLICATA'], -8), 8, '0', STR_PAD_LEFT)  . str_pad($duplicata[0]['NRPARCELA'], 2, '0', STR_PAD_LEFT);
      }
      
      //Descrição: Brancos
      //Posição: 73 a 77
      //Tamanho: X(5)
      $arquivo .= self::preenche_brancos('', 5);
      
      //Descrição: Data de Vencimento do Título
      //Posição: 78 a 85
      //Tamanho: X(8)
      $vencimento = DateUtil::getDataDeString(DateUtil::truncateString($dtvenbolrec));
      $data = DateUtil::getDia($vencimento) . DateUtil::getMes($vencimento) . DateUtil::getAno($vencimento);
      $arquivo .= Util::campoCnabValido($data);
      
      //Descrição: Valor Nominal do Título
      //Posição: 86 a 100
      //Tamanho: X(13)V(2)
      $vrfinabolrec = str_pad(($valorfinalboleto * 100), 15, "0", STR_PAD_LEFT);
      $arquivo .= Util::campoCnabValido($vrfinabolrec);
      
      //Descrição: Agência Encarregada da Cobrança
      //Posição: 101 a 105
      //Tamanho: X(5)
      //Descrição: Dígito Verificador da Agência
      //Posição: 106 a 106
      //Tamanho: X(1)
      $cdagencia = str_pad($cdagenciacnab,5);
      if($cddiaprotbol != '3')
          $arquivo = str_pad($arquivo, 106, "0");

      elseif((bool) $cdagencia !== false)
          $arquivo .= str_pad(Util::campoCnabValido($cdagencia),6,"0",STR_PAD_LEFT);

      else
          $arquivo = str_pad($arquivo, 106, "0");
       
      //Descrição: Espécie do Título
      //Posição: 107 a 108
      //Tamanho: X(2)
      // Domínio: '02' = DM Duplicata Mercantil - '04' = DS Duplicata de Serviço - ‘31’ = CC Cartão de Crédito
      $arquivo .= self::ESPECIE_TITULO;
      
      //Descrição: Identific. de Título Aceito/Não Aceito
      //Posição: 109 a 109
      //Tamanho: X(1)
      //'A' = Aceite - 'N' = Não Aceite
      $arquivo .= self::ID_TITULO_ACEITO;
      
      //Descrição: Data da Emissão do Título
      //Posição: 110 a 117
      //Tamanho: X(8)
      $now = DateUtil::getDataAtual();
      $data = DateUtil::getDia($now) . DateUtil::getMes($now) . DateUtil::getAno($now);
      $arquivo .= Util::campoCnabValido($data);
      
      //Descrição: Código do Juros de Mora
      //Posição: 118 a 118
      //Tamanho: X(1)
      
      //Descrição: Data do Juros de Mora
      //Posição: 119 a 126
      //Tamanho: X(8)
      
      //Descrição: Juros de Mora por Dia/Taxa
      //Posição: 127 a 141
      //Tamanho: X(13)V(2)
      
      $vrpemorbrec = $valorpemobrec;
        $vrmordiabrec = $valormordiabrec;
        $vrpemulbrec = $valorpemulbrec;
        $vrmultabrec = $valormultabrec;

        if (($vrpemorbrec == 0) or ((bool) $vrpemorbrec == false)){
              if (($vrpemulbrec == 0) or ((bool) $vrpemulbrec == false)){
                      if (($vrmordiabrec == 0) or ((bool) $vrmordiabrec == false)){
                              if (($vrmultabrec == 0) or ((bool) $vrmultabrec == false)){
                                  $arquivo .= Util::campoCnabValido('3');
                                  $arquivo = str_pad($arquivo, 126, "0");
                                  $arquivo = str_pad($arquivo, 141, "0");
                              }
                              else{
                                  $arquivo .= Util::campoCnabValido('2');
                                  $arquivo .= Util::campoCnabValido($vencimento);
                                  $vr = ($valormultabrec * 100 / $valorfinalbrec * 30) * 100000;
                                  $vr = str_pad($vr, 15, "0", STR_PAD_LEFT);
                                  $arquivo .= Util::campoCnabValido($vr);
                              }
                      }
                      else{
                          $arquivo .= Util::campoCnabValido('2');
                          $arquivo .= Util::campoCnabValido($vencimento);
                          $vr = ($vrpemulbrec * 30) * 100000;
                          $vr = str_pad($vr, 15, "0", STR_PAD_LEFT);
                          $arquivo .= Util::campoCnabValido($vr);
                      }
              }
              else{
                  $arquivo .= Util::campoCnabValido('1');
                  $arquivo .= Util::campoCnabValido($vencimento);
                  $vr = $vrmordiabrec * 100;
                  $vr = str_pad($vr, 15, "0", STR_PAD_LEFT);
                  $arquivo .= Util::campoCnabValido($vr);
              }
        }
        else{
            $arquivo .= Util::campoCnabValido('2');
            $arquivo .= Util::campoCnabValido($vencimento);
            $vr = $vrpemorbrec * 100000;
            $vr = str_pad($vr, 15, '0', STR_PAD_LEFT);
            $arquivo .= Util::campoCnabValido($vr);
        }
      
      //Descrição: Código do Desconto 1
      //Posição: 142 a 142
      //Tamanho: X(1)
      $arquivo .= self::CODIGO_DESCONTO;
      
      //Descrição: Data do Desconto 1
      //Posição: 143 a 150
      //Tamanho: X(8)
      $arquivo .= str_repeat('0',8);
      
      //Descrição: Valor/Percentual a ser Concedido
      //Posição: 151 a 165
      //Tamanho: X(13)V(2)
      $arquivo .= str_repeat('0',15);
      
      //Descrição: Valor do IOF a ser Recolhido
      //Posição: 166 a 180
      //Tamanho: X(13)V(2)
      $arquivo .= str_repeat('0',15);
      
      //Descrição: Valor do Abatimento
      //Posição: 181 a 195
      //Tamanho: X(13)V(2)
      $arquivo .= str_repeat('0',15);
       
      //Descrição: Identificação do Título na Empresa
      //Posição: 196 a 220
      //Tamanho: X(25)
      $arquivo .= Util::campoCnabValido(str_pad($cdfilial, 4) . str_pad($nrlanbolrec, 10) . 'CART' . str_pad($idtipocobranca.$cdcartcobbol, 7));
      
      //Descrição: Código para Protesto
      //Posição: 221 a 221
      //Tamanho: X(1)
      
      /** @var $boletreceber Boletreceber */
      $stcddiaprot = '3';
      $stqtdiaprot = '00';
      if((bool) $nrlanbolrec !== false){
              $boletreceber = Boletreceber::findOneByNrorgAndNrlanbolrec($nrorg, $nrlanbolrec);
              if((bool) $boletreceber->getCddiaprotbol() === false){
                    $stcddiaprot = '3';
              }
              else{
                    $stcddiaprot = $boletreceber->getCddiaprotbol();
              }
              if((bool) $boletreceber->getQtdiaprotbol() === false){
                    $stqtdiaprot = '00';
              }
              else{
                    $stqtdiaprot = $boletreceber->getQtdiaprotbol();
                            if($stqtdiaprot != '00')
                                    $stcddiaprot = '1';
              }
              if (($stcddiaprot = '3') or ($stqtdiaprot = '00')){
                    $stcddiaprot = '3';
                    $stqtdiaprot = '00';
              }
      }
      $arquivo .= Util::campoCnabValido($stcddiaprot);
      
      //Descrição: Número de Dias para Protesto
      //Posição: 222 a 223
      //Tamanho: X(2)
      $arquivo .= str_repeat('0',2);
      
      //Descrição: Código para Baixa/Devolução
      //Posição: 224 a 224
      //Tamanho: X(1)
      $arquivo .= Util::campoCnabValido(self::CODIGO_BAIXA);
     
      //Descrição: Número de Dias para Baixa/Devolução
      //Posição: 225 a 227
      //Tamanho: X(3)
      $arquivo .= str_repeat('0',3);
      
      //Descrição: Código da Moeda
      //Posição: 228 a 229
      //Tamanho: X(2)
      $arquivo .= Util::campoCnabValido(self::CODIGO_MOEDA);

      //Descrição: Nº do Contrato da Operação de Créd.
      //Posição: 230 a 239
      //Tamanho: X(10)
      $arquivo .= str_repeat('0',10);

      //Descrição: Uso Exclusivo FEBRABAN/CNAB
      //Posição: 240 a 240
      //Tamanho: X(1)
      $arquivo .= self::preenche_brancos('', 1). "\r\n";
      
      fprintf($file, "%s", $arquivo);
      fclose($file);
    }

    /**
     * Gera Detalhe  Segmento Q
     * @param $caminho
     * @param $nmarquivo
     * @param $nrorg
     * @param $cdbanco
     * @param $cdcliente
     * @param $idoperacao
     * @param $cdempresa
     * @param $cdfilial
     * @param $nrlanbolrec
     * @return mixed|void
     */
    public function geraDetalheSegmentoQ(
        $caminho, $nmarquivo, $nrorg, $cdbanco, $cdcliente, $idoperacao, $cdempresa, $cdfilial, $nrlanbolrec, $contador = NULL
    ){
      if (!is_dir($caminho))
          mkdir($caminho, "0777", true);
      $nmarquivo = $caminho.$nmarquivo;
      $file = fopen($nmarquivo, "a+");
      
      //Descrição: Código do Banco na Compensação
      //Posição: 1 a 3
      //Tamanho: X(3)
      $banco = Banco::getObjectByWhere('CDBANCO = ? AND NRORG = ?', array($cdbanco, $nrorg));
      $arquivo = $banco->getCdbanccnab();
      
      //Descrição: Lote de Serviço
      //Posição: 4 a 7
      //Tamanho: X(4)
      $arquivo .= Util::campoCnabValido(self::LOTE_SERVICO_2);
      
      //Descrição: Tipo de Registro
      //Posição: 8 a 8
      //Tamanho: X(1)
      $arquivo .= Util::campoCnabValido(self::IDENTIFICACAO_REGISTRO_Q);
      
      //Descrição: Nº Sequencial do Registro no Lote
      //Posição: 9 a 13
      //Tamanho: X(5)
      $contador = str_pad($contador, 5, "0", STR_PAD_LEFT);
      $arquivo .= Util::campoCnabValido($contador);
      
      //Descrição: Cód. Segmento do Registro Detalhe
      //Posição: 14 a 14
      //Tamanho: X(1)
      $arquivo .= Util::campoCnabValido(self::IDENTIFICACAO_SEGMENTO_REGISTRO_Q);
      
      //Descrição: Uso Exclusivo FEBRABAN/CNAB
      //Posição: 15 a 15
      //Tamanho: X(1)
      $arquivo .= self::preenche_brancos('', 1);
      
      //Descrição: Código de Movimento Remessa
      //Posição: 16 a 17
      //Tamanho: X(2)
     $idopecnabrec = $idoperacao;
      if((bool) $idopecnabrec !== false){
        if($idopecnabrec == 'I')
            $arquivo .= Util::campoCnabValido(self::ID_OPERACAO_CNAB_INCLUSAO); //Inclusão
        else if($idopecnabrec == 'E')
            $arquivo .= Util::campoCnabValido(self::ID_OPERACAO_CNAB_EXCLUSAO); //Exclusão
        else
          $arquivo .= Util::campoCnabValido(self::ID_OPERACAO_CNAB_ALTERACAO); //Alteração
      }
      
      //Descrição: Tipo de Inscrição
      //Posição: 18 a 18
      //Tamanho: X(1)
      /** @var $boleto Boletreceber */
      $cliente = Cliente::findOneByNrorgAndCdcliente($nrorg, $cdcliente);
        if((bool)$cdcliente === true){
            $nrinsjurclie = Util::removeMascaraInscricao($cliente->getNrinsjurclie());
        }
        if (strlen($nrinsjurclie) == '14'){
           $stTpInscCedente = self::TIPO_INSCRICAO_CEDENTE_CNPJ; //CNPJ
        }
        else{
           $stTpInscCedente = self::TIPO_INSCRICAO_CEDENTE_CPF; //CPF
        }
      $arquivo .= Util::campoCnabValido($stTpInscCedente);
      
      //Descrição: Número de Inscrição
      //Posição: 19 a 33
      //Tamanho: X(15)
      if((bool) $nrinsjurclie !== false){
          $arquivo .= str_pad($nrinsjurclie, 15, "0", STR_PAD_LEFT);
      }
      else{
          $arquivo .= str_repeat('0',15);
      }
      
      //Descrição: Nome
      //Posição: 34 a 73
      //Tamanho: X(40)
      if($cdcliente){
          $copia_nmrazsoc = $cliente->getNmrazsocclie();
          $nmrazsocclie = Util::campoCnabValido($copia_nmrazsoc);
          $arquivo .= str_pad(substr($nmrazsocclie, 0, 40) ,40);
      }
      else{
          $arquivo .= self::preenche_brancos('', 40);
      }
      
      //Descrição: Endereço
      //Posição: 74 a 113
      //Tamanho: X(40)
      $endeclie = Endeclie::findOneByNrorgAndCdclienteAndIdtpendeclie($nrorg, $cdcliente, 'C');
      $dsendeclie = Util::campoCnabValido($endeclie->getDsendeclie());
      if((bool) $dsendeclie !== false){
          $arquivo .= str_pad(substr($dsendeclie, 0, 40), 40);
      }
      else{
          $arquivo .= self::preenche_brancos('', 40);
      }
      
      //Descrição: Bairro
      //Posição: 114 a 128
      //Tamanho: X(15)
      $nmbairclie = str_pad(Util::campoCnabValido(trim($endeclie->getNmbairclie())), 15);
      if((bool) $nmbairclie !== false){
          $arquivo .= substr($nmbairclie, 0, 15);
      }
      else{
          $arquivo .= self::preenche_brancos('', 15);
      }
      
      //Descrição: CEP
      //Posição: 129 a 133
      //Tamanho: X(5)

      //Descrição: Sufixo do CEP
      //Posição: 134 a 136
      //Tamanho: X(3)
      $nrcepclie = str_pad(str_replace(" ", null, $endeclie->getNrcepclie()), 8, "0");
      if((bool) $nrcepclie !== false){
          $arquivo .= Util::campoCepTel($nrcepclie);
      }
      else{
          $arquivo .= self::preenche_brancos('', 3);
      }
      
      //Descrição: Cidade
      //Posição: 137 a 151
      //Tamanho: X(15)
      $municipio = Municipio::findOneByNrorgAndCdmunicipio($nrorg, $endeclie->getCdmunicipio());
      $nmmunicipio = substr($municipio->getNmmunicipio(), 0, 15);
      if((bool) $nmmunicipio !== false){
          $arquivo .= Util::campoCnabValido(str_pad($nmmunicipio, 15));
      }
      else{
          $arquivo .= self::preenche_brancos('', 15);
      }
      
      //Descrição: Unidade da Federação
      //Posição: 152 a 153
      //Tamanho: X(2)
      $sgestado = str_pad($endeclie->getSgestado(), 2);
      if((bool) $sgestado !== false){
          $arquivo .= Util::campoCnabValido($sgestado);
      }
      else{
          $arquivo .= self::preenche_brancos('', 15);
      }
      
      //Descrição: Tipo de Inscrição
      //Posição: 154 a 154
      //Tamanho: X(1)
      $arquivo .= str_repeat('0',1);

      //Descrição: Número de Inscrição
      //Posição: 155 a 169
      //Tamanho: X(15)
      $arquivo .= str_repeat('0',15);
      
      //Descrição: Nome do Sacador/Avalista
      //Posição: 170 a 209
      //Tamanho: X(40)
      $arquivo .= self::preenche_brancos('', 40);
      
      //Descrição: Cód. Bco. Corresp. na Compensação
      //Posição: 210 a 212
      //Tamanho: X(3)
      $arquivo .= str_repeat('0',3);
      
      //Descrição: Nosso Nº no Banco Correspondente
      //Posição: 213 a 232
      //Tamanho: X(20)
       $arquivo .= self::preenche_brancos('', 20);
       
      //Descrição: Uso Exclusivo FEBRABAN/CNAB
      //Posição: 233 a 240
      //Tamanho: X(8)
      $arquivo .= self::preenche_brancos('', 8) . "\r\n";

      fprintf($file, "%s", $arquivo);
      fclose($file);
    }

    /**
     * Gera Detalhe Segmento R
     * @param $caminho
     * @param $nmarquivo
     * @param $nrorg
     * @param $cdbanco
     * @param $idoperacao
     * @param $dtvenbolrec
     * @param $valorpemulbrec
     * @param $nrlanbolrec
     * @param $valormultabrec
     * @return mixed|void
     */
    public function geraDetalheSegmentoR(
        $caminho, $nmarquivo, $nrorg, $cdbanco, $idoperacao, $dtvenbolrec, $valorpemulbrec, $nrlanbolrec, $valormultabrec, $contador = NULL
    ){
      
      if (!is_dir($caminho))
      mkdir($caminho, "0777", true);
      $nmarquivo = $caminho.$nmarquivo;
      $file = fopen($nmarquivo, "a+");
        
      //Descrição: Código do Banco na Compensação
      //Posição: 1 a 3
      //Tamanho: X(3)
      $banco = Banco::getObjectByWhere('CDBANCO = ? AND NRORG = ?', array($cdbanco, $nrorg));
      $arquivo = $banco->getCdbanccnab();
      
      //Descrição: Lote de Serviço
      //Posição: 4 a 7
      //Tamanho: X(4)
      $arquivo .= self::LOTE_DO_SERVICO;
      
      //Descrição: Tipo de Registro
      //Posição: 8 a 8
      //Tamanho: X(1)
      $arquivo .= Util::campoCnabValido(self::IDENTIFICACAO_REGISTRO_R);
      
      //Descrição: Nº Sequencial do Registro no Lote
      //Posição: 9 a 13
      //Tamanho: X(5)
      $contador = str_pad($contador, 5, "0", STR_PAD_LEFT);
      $arquivo .= Util::campoCnabValido($contador);
      
      //Descrição: Cód. Segmento do Registro Detalhe
      //Posição: 14 a 14
      //Tamanho: X(1)
      $arquivo .= Util::campoCnabValido(self::IDENTIFICACAO_SEGMENTO_REGISTRO_R);
      
      //Descrição: Uso Exclusivo FEBRABAN/CNAB
      //Posição: 15 a 15
      //Tamanho: X(1)
      $arquivo .= self::preenche_brancos('', 1);
      
      //Descrição: Código de Movimento Remessa
      //Posição: 16 a 17
      //Tamanho: X(2)
      $idopecnabrec = $idoperacao;
      if((bool) $idopecnabrec !== false){
        if($idopecnabrec == 'I')
                $arquivo .= Util::campoCnabValido(self::ID_OPERACAO_CNAB_INCLUSAO); //Inclusão
        else if($idopecnabrec == 'E')
                $arquivo .= Util::campoCnabValido(self::ID_OPERACAO_CNAB_EXCLUSAO); //Exclusão
        else
                $arquivo .= Util::campoCnabValido(self::ID_OPERACAO_CNAB_ALTERACAO); //Alteração
      }
      
      //Descrição: Código do Desconto 2
      //Posição: 18 a 18
      //Tamanho: X(1)
      $arquivo .= self::CODIGO_DESCONTO;
      
      //Descrição: Data do Desconto 2
      //Posição: 19 a 26
      //Tamanho: X(8)
      $arquivo .= str_repeat('0',8);
      
      //Descrição: Valor/Percentual a ser Concedido
      //Posição: 27 a 41
      //Tamanho: X(13)V(2)
      $arquivo .= str_repeat('0',15);
      
      //Descrição: Código do Desconto 3
      //Posição: 42 a 42
      //Tamanho: X(1)
      $arquivo .= self::CODIGO_DESCONTO;
      
      //Descrição: Data do Desconto 3
      //Posição: 43 a 50
      //Tamanho: X(8)
      $arquivo .= str_repeat('0',8);
      
      //Descrição: Valor/Percentual a Ser Concedido
      //Posição: 51 a 65
      //Tamanho: X(13)V(2)
      $arquivo .= str_repeat('0',15);
      
      //Descrição: Código da Multa
      //Posição: 66 a 66
      //Tamanho: X(1)
      // '0' = Não Registra a Multa - '2' = Percentual
      // '3' = Acatar parâmetro do convênio
      // ‘4’ = Valor da Multa
      $vrpemulbrec = $valorpemulbrec;
      if($valorpemulbrec != '0')
            $arquivo .= '2'; 
        else if ($valormultabrec != '0')
            $arquivo .= '4'; 
        else
            $arquivo .= '0'; 
            
      //Descrição: Data da Multa
      //Posição: 67 a 74
      //Tamanho: X(8)
      if($valormultabrec == 0 && $valorpemulbrec == 0)
          $arquivo .= str_repeat('0',8);
      else{
          $vencimento = DateUtil::getDataDeString(DateUtil::truncateString($dtvenbolrec));
          $vencimento = DateUtil::adicionaIntervalo($vencimento, 1);
          $data = DateUtil::getDia($vencimento) . DateUtil::getMes($vencimento) . DateUtil::getAno($vencimento);
          $arquivo .= Util::campoCnabValido($data);
      }
      
      //Descrição: Percentual a Ser Aplicado
      //Posição: 75 a 89
      //Tamanho: X(13)V(2)
      $vrmultabrec = $valormultabrec;
        if (($vrpemulbrec == 0) or ((bool) $vrpemulbrec === false)){
            if (($vrmultabrec == 0) or ((bool) $vrmultabrec === false)){
                $arquivo = str_pad($arquivo, 89, "0");
            }
            else{
                $vrmultabrec = $vrmultabrec * 100;
                $arquivo .= Util::campoCnabValido(str_pad($vrmultabrec, 15, '0', STR_PAD_LEFT));
            }
        }
        else{
            $vrpemulbrec = $vrpemulbrec * 100;
            $arquivo .= Util::campoCnabValido(str_pad($vrpemulbrec, 15, '0', STR_PAD_LEFT));
        }
      
      //Descrição: Informação ao Sacado
      //Posição: 90 a 99
      //Tamanho: X(10)
      $arquivo .= self::preenche_brancos('', 10);
      
      //Descrição: Mensagem 3
      //Posição: 100 a 139
      //Tamanho: X(40)
      $boletreceber = Boletreceber::findOneByNrorgAndNrlanbolrec($nrorg, $nrlanbolrec);
      if((bool)$boletreceber->getTxmensbolbol() === false){
          $arquivo .= self::preenche_brancos('',40);
      }
      else{
        $arquivo .= Util::campoCnabValido($boletreceber->getTxmensbolbol());
      }
      
      //Descrição: Mensagem 4
      //Posição: 139 a 179
      //Tamanho: X(40)
      $arquivo .= self::preenche_brancos('',40);
      
      //Descrição: Uso Exclusivo FEBRABAN/CNAB
      //Posição: 180 a 199
      //Tamanho: X(20)
      $arquivo .= self::preenche_brancos('',20);
      
      //Descrição: Cód. Fornecido pelo Banco
      //Posição: 200 a 202
      //Tamanho: X(3)
      $arquivo .= str_repeat('0',3);
      
      //Descrição: Cód. Ocor. do Sacado
      //Posição: 203 a 207
      //Tamanho: X(5)
      $arquivo .= str_repeat('0',5);
      
      //Descrição: Cód. do Banco na Conta do Débito
      //Posição: 208 a 210
      //Tamanho: X(3)
      $arquivo .= str_repeat('0',3);
      
      //Descrição: Código da Agência do Débito
      //Posição: 211 a 215
      //Tamanho: X(5)
      $arquivo .= str_repeat('0',5);
      
      //Descrição: Dígito Verificador da Agência
      //Posição: 216 a 216
      //Tamanho: X(1)
      $arquivo .= str_repeat('0',1);
      
      //Descrição: Conta Corrente para Débito
      //Posição: 217 a 228
      //Tamanho: X(12)
      $arquivo .= str_repeat('0',12);
      
      //Descrição: Dígito Verificador da Conta
      //Posição: 229 a 229
      //Tamanho: X(1)
      $arquivo .= str_repeat('0',1);
      
      //Descrição: Dígito Verificador Ag/Conta
      //Posição: 230 a 230
      //Tamanho: X(1)
      $arquivo .= str_repeat('0',1);
      
      //Descrição: Aviso para Débito Automático
      //Posição: 231 a 231
      //Tamanho: X(1)
      $arquivo .= self::preenche_brancos('', 1);
      
      //Descrição: Uso Exclusivo FEBRABAN/CNAB
      //Posição: 232 a 240
      //Tamanho: X(9)
      $arquivo .= self::preenche_brancos('', 9). "\r\n";
      
      fprintf($file, "%s", $arquivo);
      fclose($file);
      
    }

    /**
     * Gera Trailler Lote
     * @param $caminho
     * @param $nmarquivo
     * @param $nrorg
     * @param $cdbanco
     * @param null $contreg
     * @return mixed|void
     */
    public function geraTraillerLote(
        $caminho, $nmarquivo, $nrorg, $cdbanco, $contreg = NULL
    ){
      
      $nmarquivo = $caminho.$nmarquivo;
      $file = fopen($nmarquivo, "a+");
      
      //Descrição: Código do Banco na Compensação
      //Posição: 1 a 3
      //Tamanho: X(3)
      $banco = Banco::getObjectByWhere('CDBANCO = ? AND NRORG = ?', array($cdbanco, $nrorg));
      $arquivo = $banco->getCdbanccnab();
        
      //Descrição: Lote de Serviço
      //Posição: 4 a 7
      //Tamanho: X(4)
      $arquivo .= self::LOTE_DO_SERVICO;
      
      //Descrição: Tipo de Registro
      //Posição: 8 a 8
      //Tamanho: X(1)
      $arquivo .= self::TIPO_REGISTRO_TRAILLER_LOTE;
      
      //Descrição: Uso Exclusivo FEBRABAN/CNAB
      //Posição: 9 a 17
      //Tamanho: X(9)
      $arquivo .= self::preenche_brancos('', 9);
       
      //Descrição: Quantidade de Registros no Lote
      //Posição: 18 a 23
      //Tamanho: X(6)
      $arquivo .= Util::campoCnabValido(str_pad($contreg - 1, 6, "0", STR_PAD_LEFT));
      
      //Descrição: Quantidade de Títulos em Cobrança
      //Posição: 24 a 29
      //Tamanho: X(6)
      $arquivo = str_pad($arquivo, 29, '0');
      
      //Descrição: Valor Total dosTítulos em Carteiras
      //Posição: 30 a 46
      //Tamanho: X(15)V(2)
      $arquivo = str_pad($arquivo, 46, '0');
      
      //Descrição: Quantidade de Títulos em Cobrança
      //Posição: 47 a 52
      //Tamanho: X(6)
      $arquivo = str_pad($arquivo, 52, '0');
      
      //Descrição: Valor Total dos Títulos em Carteiras
      //Posição: 53 a 69
      //Tamanho: X(15)V(2)
      $arquivo = str_pad($arquivo, 69, '0');
      
      //Descrição: Quantidade de Títulos em Cobrança
      //Posição: 70 a 75
      //Tamanho: X(6)
      $arquivo = str_pad($arquivo, 75, '0');
      
      //Descrição: Quantidade de Títulos em Carteiras
      //Posição: 76 a 92
      //Tamanho: X(15)V(2)
      $arquivo = str_pad($arquivo, 92, '0');
      
      //Descrição: Quantidade de Títulos em Cobrança
      //Posição: 93 a 98
      //Tamanho: X(6)
     $arquivo = str_pad($arquivo, 98, '0');
      
      //Descrição: Valor Total dosTítulos em Carteiras
      //Posição: 99 a 115
      //Tamanho: X(15)V(2)
      $arquivo = str_pad($arquivo, 115, '0');
      
      //Descrição: Número do Aviso de Lançamento
      //Posição: 116 a 123
      //Tamanho: X(8)
      $arquivo = str_pad($arquivo, 123);
      
      //Descrição: Uso Exclusivo FEBRABAN/CNAB
      //Posição: 124 a 240
      //Tamanho: X(117)
      $arquivo = str_pad($arquivo, 240) . "\r\n";
      
      fprintf($file, "%s", $arquivo);
      fclose($file);
      
    }

    /**
     * Gera Trailler
     * @param $caminho
     * @param $nmarquivo
     * @param $nrorg
     * @param $cdbanco
     * * @param null $contreg
     * @return mixed|void
     */
    public function geraTrailler(
        $caminho, $nmarquivo, $nrorg, $cdbanco, $contlote = NULL, $contreg = NULL, $cdreducont = NULL
    ){
      if (!is_dir($caminho))
        mkdir($caminho, "0777", true);
        $nmarquivo = $caminho.$nmarquivo;
        $file = fopen($nmarquivo, "a+");
      
      //Descrição: Código do Banco na Compensação
      //Posição: 1 a 3
      //Tamanho: X(3)
      $banco = Banco::getObjectByWhere('CDBANCO = ? AND NRORG = ?', array($cdbanco, $nrorg));
      $arquivo = $banco->getCdbanccnab();
      
      //Descrição: Lote de Serviço
      //Posição: 4 a 7
      //Tamanho: X(4)
      $arquivo .= Util::campoCnabValido(self::LOTE_SERVICO_TRAILLER);
      
      //Descrição: Tipo de Registro
      //Posição: 8 a 8 
      //Tamanho: X(1)
      $arquivo .= Util::campoCnabValido(self::IDENTIFICACAO_ARQUIVO_TRAILLER);
      
      //Descrição: Uso Exclusivo FEBRABAN/CNAB
      //Posição: 9 a 17 
      //Tamanho: X(9)
      $arquivo .= self::preenche_brancos('', 9);
      
      //Descrição: Quantidade de Lotes do Arquivo
      //Posição: 18 a 23 
      //Tamanho: X(6)
      $arquivo .= Util::campoCnabValido(str_pad($contlote, 6, "0", STR_PAD_LEFT));
      
      //Descrição: Quantidade de Registros do Arquivo
      //Posição: 24 a 29 
      //Tamanho: X(6)
      $arquivo .= Util::campoCnabValido(str_pad($contreg, 6, "0", STR_PAD_LEFT));
      
      //Descrição: Qtde de Contas p/ Conc. (Lotes)
      //Posição: 30 a 35 
      //Tamanho: X(6)
      $arquivo = str_pad($arquivo, 35, '0');
      
      //Descrição: Uso Exclusivo FEBRABAN/CNAB
      //Posição: 36 a 240 
      //Tamanho: X(205)
      $arquivo = str_pad($arquivo, 240) . "\r\n";
      
      fprintf($file, "%s", $arquivo);
      fclose($file);
    }
    
    /**
     * @param $valor - valor a ser alterado
     * @param $tamanho - tamanho total da string desejado
     * @return string
    */
     
    private function preenche_brancos($valor, $tamanho){
        return str_pad($valor, $tamanho);
    }

    /**
     * Gera Transação
     * @param $caminho
     * @param $nmarquivo
     * @param $cdempresa
     * @param $cdcartcobbol
     * @param $cdagencia
     * @param $cdcontcorr
     * @param $cdfilial
     * @param $nrlanbolrec
     * @param $vrpemobrec
     * @param $idimprboleto
     * @param $cdnosnumrec
     * @param $dtvenbolrec
     * @param $cdins1cobbol
     * @param $qtdiaprotbol
     * @param $vrmordiabrec
     * @param $vrfinalbrec
     * @param $idopecnabrec
     * @param $vrpemulbrec
     * @param $idtpijurclie
     * @param $nrinsjurclie
     * @param $cdcliente
     * @param $cdins2cobbol
     * @return mixed|void
     */
    public function geraTransacao(
        $caminho, $nmarquivo,$nrorg, $cdempresa, $cdcartcobbol, $cdagencia, $cdcontcorr, $cdfilial,
        $nrlanbolrec, $vrpemobrec, $idimprboleto, $cdnosnumrec, $dtvenbolrec, $cdins1cobbol,
        $qtdiaprotbol, $vrmordiabrec, $vrfinalbrec, $idopecnabrec, $vrpemulbrec, $idtpijurclie,
        $nrinsjurclie, $cdcliente, $cdins2cobbol, $contreg = NULL, $idtipcnabrec = null, $idfilialcnabrec, $cddiaprotbol = NULL, $contador = NULL, $cdreducont,$cdbanco
    ){

    }
}