<?php

namespace Teknisa\FIN\Modules\MovimentacaoBancaria\Controllers;


use Teknisa\Libs\Service\CRUD as CrudService;
use Teknisa\Libs\Util\Environment;
use Teknisa\FIN\Controllers\TeknisaCrud;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO\Request\DataSet as Data;
use Teknisa\FIN\Modules\MovimentacaoBancaria\Services\Bco56200LancamentoDespesaService;

/**
 * Class Bco56200LancamentoDespesaController
 * @package Teknisa\FIN\Modules\MovimentacaoBancaria\Controllers
 */
class Bco56200LancamentoDespesaController extends TeknisaCrud {

    protected $crudService, $parameterBag, $environment, $bco56200LancamentoDespesaService;

    /**
     * @param CrudService  $crudService
     * @param ParameterBag $parameterBag
     * @param Environment  $environment
     * @param Bco56200LancamentoDespesaService $bco56200LancamentoDespesaService
     */
    public function __construct(CrudService $crudService, ParameterBag $parameterBag, Environment $environment, Bco56200LancamentoDespesaService $bco56200LancamentoDespesaService)
    {
        $this->crudService  = $crudService;
        $this->parameterBag = $parameterBag;
        $this->environment  = $environment;
        $this->bco56200LancamentoDespesaService = $bco56200LancamentoDespesaService;
    }
    
    /**
     * Salva Lançamento de Despesa Caixa
     * @param Row $row
     * @param Response $response
     */
    public function saveLancDespCaixa(Row $row, Response $response)
    {
        try {
            $row = $row->getRow();

            $this->bco56200LancamentoDespesaService->saveLancDespCaixa($row, $response);

            $response->addDataSet(new DataSet('saveLancDespCaixa', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Deleta Lançamento de Despesa Caixa
     * @param Row $row
     * @param Response $response
     */
    public function deleteLancDespCaixa(Row $row, Response $response)
    {
        try {
            $row = $row->getRow();

            $this->bco56200LancamentoDespesaService->deleteLancDespCaixa($row, $response);

            $response->addDataSet(new DataSet('deleteLancDespCaixa', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
}
