<?php

/**
 * GepObservacaosoliLogic.
 *
 * @package Logic.
 * @author  erikasouza
 *
 */
class GepObservacaosoliLogic{


    private $_GepObservacaosoliEntity;


    /**
     * Construtor da classe.
     */
    public function __construct(){
        $this->_GepObservacaosoliEntity = new GepObservacaosoliEntity();
    }


    /**
     * Novo codigo GepObservacaosoli.
     *
     * @param integer $nrorg Numero da organização.
     *
     * @return Ambigous <Ambigous, number, number, string>
     */
    public static function novoCodigoGepObaservacaoSoli($nrorg){
        $novo_codigo = GepObservacaosoliEntity::novoCodigoGepObaservacaoSoli($nrorg);
        return $novo_codigo;
    }


    /**  ok
     * Retorna observacoes da solicitacao por parceiro.
     *
     * @param integer $nrorg       Numero da organizacao.
     * @param integer $solicitacao Numero da solicitacao.
     * @param integer $parcnegocio Parceiro de negocio.
     *
     * @return GepObservacaosoliEntity|boolean
     */
    public function retornaObservacoesSoliParceiro($nrorg, $solicitacao, $parcnegocio){
        try {
            $dados = $this->_GepObservacaosoliEntity->retornaObservacoesSoliParceiro(
                $nrorg, $solicitacao, $parcnegocio
            );
            return $dados;
        } catch (Exception $e) {
            switch ($e->getCode()) {
            case GepObservacaosoliEntity::SEM_REGISTRO:
                return GepObservacaosoliEntity::SEM_REGISTRO;
            default:
                tk_warning('Erro desconhecido: '.$e->getMessage());
                return false;
            }
        }
    }


    /**     ok
     * Retorna consulta da observacao geral de uma solicitacao.
     *
     * @param integer $nrorg       Numero da organizacao.
     * @param integer $solicitacao Numero da solicitacao.
     *
     * @return GepObservacaosoliEntity|boolean
     */
    public function retornaObservacaoGeralSoli($nrorg, $solicitacao){
        try {
            $dados = $this->_GepObservacaosoliEntity->retornaObservacaoGeralSoli(
                $nrorg, $solicitacao
            );
            return $dados;
        } catch (Exception $e) {
            switch ($e->getCode()) {
            case GepObservacaosoliEntity::SEM_REGISTRO:
                return GepObservacaosoliEntity::SEM_REGISTRO;
            default:
                tk_warning('Erro desconhecido: '.$e->getMessage());
                return false;
            }
        }
    }


    /**        ok
     * Salva observacoes (geral e do operador) de uma solicitacao.
     *
     * @param integer $nrorg       Numero da organizacao.
     * @param integer $solicitacao Codigo da solicitacao.
     * @param integer $parceiro    Numero do parciero de negocio.
     * @param string  $txobs_soli  Texto da observacao do operador.
     * @param string  $txobs_geral Texto da observacao geral.
     * @param date    $data        Data.
     * @param integer $operador    Codigo do operador.
     *
     * @return boolean
     */
    public function salvaObservacoes(
        $nrorg, $solicitacao, $parceiro, $txobs_soli, $txobs_geral, $data, $operador
    ){
        // Observacao por parceiro.
        $observ_parc= $this->retornaObservacoesSoliParceiro(
            $nrorg, $solicitacao, $parceiro
        );
        if ($observ_parc === false OR $observ_parc === GepObservacaosoliEntity::SEM_REGISTRO) {
            if (empty($txobs_soli) === false) {
                $codigo = $this->_GepObservacaosoliEntity->novoCodigoGepObaservacaoSoli($nrorg);
                $insere_obs = $this->_GepObservacaosoliEntity->insereObservacaoSoli(
                    $codigo, $solicitacao, $parceiro, $txobs_soli, $nrorg, $data, $operador
                );
            }
        } else {
            $strSqparceiro = 'NRPARCNEGOCIO = '.$parceiro;
            $atualiza_obs = $this->_GepObservacaosoliEntity->atualizaObservacaoSoli(
                $strSqparceiro, $nrorg, $solicitacao, $txobs_soli, $data, $operador
            );
        }

        // Observacao geral.
        $observ_geral = $this->retornaObservacaoGeralSoli($nrorg, $solicitacao);
        if (is_object($observ_geral) === false OR $observ_geral === GepObservacaosoliEntity::SEM_REGISTRO) {
            if (empty($txobs_geral) === false) {
                $codigo = $this->_GepObservacaosoliEntity->novoCodigoGepObaservacaoSoli($nrorg);
                $insere_obs = $this->_GepObservacaosoliEntity->insereObservacaoSoli(
                    $codigo, $solicitacao, null, $txobs_geral, $nrorg, $data, $operador
                );
            }
        } else {
            $strSqparceiro = 'NRPARCNEGOCIO IS NULL';
            $atualiza_obs = $this->_GepObservacaosoliEntity->atualizaObservacaoSoli(
                $strSqparceiro, $nrorg, $solicitacao, $txobs_geral, $data, $operador
            );
        }

        return true;
    }


    /**
     * Atualiza o valor da cor de uma observacao de um determinado parceiro e solicitacao.
     *
     * @param integer $nrorg       Numero da organizacao.
     * @param integer $parceiro    Numero do parceiro.
     * @param integer $solicitacao Numero da solicitacao.
     * @param string  $fonte       Codigo RGB da fonte.
     *
     * @return boolean
     */
    public function atualizaCorFonteObservSoli($nrorg, $parceiro, $solicitacao, $fonte){
        $atualizacao = $this->_GepObservacaosoliEntity->atualizaCorFonteObservSoli(
            $nrorg, $parceiro, $solicitacao, $fonte
        );
        return $atualizacao;
    }


}